var _ = require('lodash');
var path = require('path');

var Promise = require('bluebird');
var program = require('./util/commander');

function cli() {
    var def = this;
    return new Promise(function (resolve) {
        program
            .version(def.version)
            .option('-v, --verbose', 'output logs of all level')
            .option('-s, --silent', 'disable log output')
            .option('--no-color', 'disable log output');


        program.command('help [plugin] [command]')
            .description('show help')
            .action(function (plugin, command) {
                if (!plugin) {
                    program.help();
                } else {
                    var defPlugin = def.getPlugin(plugin);

                    if (!defPlugin) {
                        def.log.error('no command: ' + defPlugin);
                        process.exit(1);
                    }

                    if (!command) {
                        defPlugin.cli.help();
                    } else {
                        var cmd = _.find(defPlugin.cli.commands, function (c) {
                            return c._name === command;
                        });

                        if (!cmd) {
                            def.log.error('plugin ' + defPlugin + 'has no command ' + command);
                            process.exit(1);
                        }
                        cmd.help();
                    }

                }
            });
        if (process.platform === 'darwin') {
            program.command('setup')
                .description('setup def')
                .action(function () {
                    var service = require('./service');

                    return service.install(def).then(function(){
                        def.log.info('install successful!');
                    }, def.fatal);

                });
        }


        program.on('*', function () {
            program.help();
            process.exit(1);
        });


        def.request('alias:getAlias').then(function (alias) {

            var head = process.argv.slice(0, 2);
            var tail = process.argv.slice(2);

            alias = _.defaults(alias || {}, {
                install: 'plugin install',
                i: 'plugin install',
                remove: 'plugin remove',
                rm: 'plugin remove',
                list: 'plugin list',
                ls: 'plugin list',
                update: 'plugin update',
                up: 'plugin update'
            });

            _.pairs(alias).forEach(function (p) {
                var alias = p[0];
                var cmd = p[1];
                var aAlias = alias.split(' ');
                var match = true;

                aAlias.forEach(function (a, index) {
                    if (a !== tail[index]) {
                        match = false;
                    }
                });

                if (match) {
                    tail = cmd.split(/\s+/).concat(tail.splice(aAlias.length))
                }
            });

            var argv = head.concat(tail);

            var argvParsed = require('yargs').parse(argv);

            // preload plugin
            if (argvParsed._.length > 2) {
                def.getPlugin(argvParsed._[2]);
            }

            program.parse(_.without(argv, '--daemon'));

            // if no args and options, print the help
            if (process.argv.length === 2) {
                program.help();
                process.exit(1);
            }

            resolve(program);

        }, function () {
            def.log.warn('core', 'alias request error');
            program.parse(_.without(process.argv, '--daemon'));
            resolve(program);
        });
    });

}

cli.program = program;

module.exports = cli;