/*lend to def*/

var dUtil = require('util-def');
var _ = require('lodash');
var path = require('path');


module.exports = function(defHome) {
    var actions = {};

    actions['plugin:pluginVersions'] = function(options, cb) {
        console.log('action', 'plugin:pluginVersions');
        dUtil.listModules(defHome).then(function(pkgs){
            var ret = {};
            pkgs.forEach(function(pkg){
                ret[pkg.name] = pkg.version;
            });
            cb(null, {
                result: ret
            });

        }, cb)
    };

    var channels = {};

    /**
     * request an plugin api
     * @param options
     * @param options.action
     * @param options.data
     * @param cb
     */
    channels.request = function(options, cb) {
        console.log('channel:request');
        if (!options.action) {
            cb({
                error: 'request channel error; no action'
            });
            return;
        }

        if (!_.has(actions, options.action)) {
            cb({
                error: 'action ' + options.action + ' was not available'
            });
            return;
        }

        actions[options.action](options.data, cb);

    };


    /**
     * request an plugin api
     * @param {Object} options
     * @param {Array} options.packages
     * @param cb
     */
    channels.update = function(options, cb) {
        var packages;

        if (options && options.packages && options.packages.length) {
            packages = options.packages;
        }

        if (!packages) {
            cb('no packages');
            return;
        }
        console.log('update');
        console.log(options);


        var args = [path.join(__dirname, '../../bin/def'), 'install'];
        args = args.concat(packages);
        args.push('--daemon', '--no-color');
        console.log(args.join(' '));
        console.log(process.env);
        console.log(process.execPath);
        var child = dUtil.spawn(process.execPath, args);

        child.on('error', function(err){
            console.log(err);
            cb(err);
        });

        var out = '';

        child.stderr.on('data', function(data){
            process.stderr.write(data);
            out += data;
        });

        child.stdout.on('data', function(data){
            process.stdout.write(data);
            out += data;
        });


        child.on('exit', function(code) {
            cb(null, {
                code: code,
                out: out
            });
        });
    };

    channels.ping = function(options, cb) {
        cb(null, 'pong');
    };

    return function(channel, options, cb) {
        if(_.has(channels, channel)) {
            channels[channel](options, cb);
        } else {
            cb('no channel');
        }
    };

};


