process.title = 'defd-main';

var Client = require('./client');

var net = require('net');
var fs = require('fs');
var os = require('os');
var path = require('path');

var JsonSocket = require('json-socket');

var pkg = JSON.parse(fs.readFileSync(path.join(__dirname, '../../package.json'), 'utf8'));

var defHome = process.env.DEF_HOME || '.';

var remoteHost = process.env.REMOTE_HOST || 'def.taobao.net';
var defVersion = pkg.version;
var remotePort = process.env.REMOTE_PORT || 51999;
var localPort = process.env.LOCAL_PORT || 38555;

var log = require('util-def').log;


log.level = 'silly';

log.info('daemon', 'start', new Date());
log.info('daemon', 'home:', defHome);
log.info('daemon', 'version:', defVersion);
log.info('daemon', 'remote port:', remotePort);
log.info('daemon', 'local port:', localPort);

var confFile = path.join(defHome, 'conf.json');
var config = (defHome && fs.existsSync(confFile)) ? JSON.parse(fs.readFileSync(confFile, 'utf8')) : {};

var local = net.createServer();

var remote = new Client({
    port: remotePort,
    host: remoteHost,
    autoReconnect: true
});


remote.on('reconnect', function(timeout){
    log.info('remote', 'reconnecting %s', timeout);
});

local.on('connection', function(socket) {
    log.info('local', '%s %s', new Date(), 'local Connected');

    socket = new JsonSocket(socket);

    socket.sendMessage({});

    socket.on('message', function(message) {
        log.info('local','message from cli', message);

        if (message._def_ver !== defVersion) {
            // if version not match, a restart can fix it
            process.exit(98);
        }

        if (message.type === 'restart') {
            // manually restart
            process.exit(97);
        }

        remote.send(message);
    });

    socket.on('close', function() {
        log.info('local', 'closed');
    });

    socket.on('error', function(err) {
        log.info('local', 'error');
        log.error('local', err);
    });

});

// port 0 = auto select port
local.listen(localPort, '127.0.0.1',  function(){
    log.info('local', 'server is listen on ' + local.address().port);
});

log.info('remote', 'connecting to ', remotePort, remoteHost);

remote.connect();

remote.on('request', remoteRequestHandler);

remote.on('type:restart', function () {
    log.info('remote', 'restart by remote');
    process.exit(99);
});

remote.on('connect', function(remote) {

    log.info('remote', 'connected', remote.options.host, remote.options.port);

    remote.on('type:heartbeat-ping', function(){
        setTimeout(function(){
            remote.send({
                type:'heartbeat-ping-back'
            });
        }, 0);

    });

    remote.send({
        type: 'auth',
        email: config.email || 'nobody@nobody.com',
        mid: config.mid || 'noMid',
        name: config.name || 'noName',
        version: defVersion,
        nodeVersion: process.version,
        platform: process.platform,
        arch: os.arch(),
        os: [os.type(), os.release()].join(' '),
        now: Date.now(),
        heartbeat: true
    });
});

remote.on('error', function(err) {
    if (err) {
        log.error('remote', err);
    }
});

var callChannels = require('./channels')(defHome);

function remoteRequestHandler(channel, options,  cb) {
    callChannels(channel, options, cb);
}