var Promise = require('bluebird');
var fs = require('fs');
var path = require('path');
var dUtil = require('util-def');
var service = module.exports;

service.getSVC = function(def, notInstall) {

    def = def || this;

    return new Promise(function(resolve, reject) {
        var pkg;

        switch (process.platform){
            case 'win32':
                pkg = 'daemon-win-def';
                break;
            case 'darwin':
                pkg = 'daemon-mac-def';
                break;
            default:
                reject('not mac or windows');
                return;
        }

        function load() {

            var childScript2 = path.join(def.home, 'node_modules/def-cli/lib/daemon/child-main.js');
            var childScript = fs.existsSync(childScript2) ? childScript2 : path.join(__dirname, 'daemon/child.js');

            var daemonScript = path.join(def.home, 'def-daemon.js');

            if (process.platform === 'win32') {
                childScript = childScript.replace(/\\/g, '\\\\');
            }

            var content = 'require(\'' + childScript + '\');\n';

            fs.writeFileSync(daemonScript, content, 'utf8');

            fs.lstat(path.join(def.home, 'node_modules', pkg), function(err, stat){
                (stat.isSymbolicLink() || notInstall ? Promise.resolve() :  dUtil.install(pkg + '@latest', def.home)).then(function(){

                    var options = {
                        name: 'DEF',
                        desc: 'The Develop environment for frontend.',
                        env: {
                            'PATH': process.env['PATH'],
                            'DEF_HOME': process.env['DEF_HOME'] || def.home,
                            'REMOTE_HOST': process.env['REMOTE_HOST'] || 'def.taobao.net',
                            'DEF_VERSION': def.version,
                            'REMOTE_PORT': process.env['REMOTE_PORT'] || 51999,
                            'LOCAL_PORT': process.env['LOCAL_PORT'] || 38555
                        }
                    };

                    if (process.platform === 'win32') {
                        options.env['USERPROFILE'] = process.env['USERPROFILE']
                    } else {
                        options.env['HOME'] = process.env['HOME']
                    }


                    var svc = require(path.join(def.home, 'node_modules', pkg, 'index.js'))(daemonScript, options);

                    resolve(svc);

                }, reject);
            });


        }

        if(fs.existsSync(path.join(def.home, 'node_modules', pkg))) {
            load();
        } else {
            dUtil.install([pkg], def.home).then(load, reject);
        }
    });

};

service.install = function(def) {
    def = def || this;

    def.log.verbose('service', 'install svc on windows');

    return new Promise(function(resolve, reject){
        service.getSVC(def)
            .then(function(svc) {

                svc.on('install', function(){
                    def.log.verbose('service', 'installed, starting');
                    svc.start();
                });

                svc.on('alreadyinstalled', function(){
                    def.log.verbose('service', 'alreadyinstalled, starting');
                    svc.start();
                });

                svc.on('start', function(){
                    def.log.verbose('service', 'started');
                    setTimeout(resolve, 1000);
                });

                if (!svc.exists) {
                    def.log.verbose('service', 'not exists, install');
                    svc.install();
                } else {
                    def.log.verbose('service', 'exists, uninstall svc');

                    svc.on('uninstall', function(){
                        def.log.verbose('service', 'uninstall success, now install it');
                        setTimeout(function(){
                            svc.install();
                        }, 1000);
                    });

                    svc.uninstall();


                }

            })
            .catch(reject);
    });
};

service.uninstall = function uninstall(def) {
    def.log.verbose('service', 'uninstall svc');
    def = def || this;
    return new Promise(function(resolve, reject){
        service.getSVC(def)
            .then(function(svc){
                if (!svc.exists) {
                    resolve();
                    return;
                }

                svc.on('uninstall', resolve);
                svc.on('error', reject);
                svc.uninstall();
            })
            .catch(reject);
    });
};