var fs = require('fs');
var path = require('path');
var rimraf = require('rimraf');
var mkdirp = require('mkdirp');
var init = require('../lib/init');
var expect = require('expect.js');
var Promise = require('bluebird');
describe('def init', function(){

    var def = {
        home: 'tmp/.def'
    };

    function initTemp(){
        var temp = path.dirname(def.home);
        rimraf.sync(temp);
        mkdirp.sync(temp);
    }

    describe('initHome', function(){

        afterEach(function(){
            rimraf.sync(path.dirname(def.home));
        });

        describe('file exists', function(){
            before(function(){
                initTemp();
                fs.writeFileSync(def.home, '');
                return init.initHome(def);
            });
            it('should remove file and create directory', function(){
                expect(fs.statSync(def.home).isDirectory()).to.be(true);
            });
        });

        describe('dir exists', function(){
            before(function(){
                initTemp();
                fs.mkdirSync(def.home);
                return init.initHome(def);
            });
            it('should keep directory', function(){
                expect(fs.statSync(def.home).isDirectory()).to.be(true);
            });
        });

        describe('dir not exits', function(){
            before(function(){
                initTemp();
                fs.writeFileSync(def.home, '');
                return init.initHome(def);
            });

            it('should create directory', function(){
                expect(fs.statSync(def.home).isDirectory()).to.be(true);
            });
        });
    });

    describe('initPkgJSON',function(){
        var def = {
            home: 'tmp/.def'
        };

        var expectPkg = {
            "name": "def-home",
            "version": "0.0.0",
            "private": true
        };

        before(function(){
            mkdirp.sync(def.home);
        });

        describe('package.json not exits', function(){

            before(function(){
                return init.initPkgJSON(def);
            });

            it('should create package.json', function(){
                expect(JSON.parse(fs.readFileSync(path.join(def.home, 'package.json')))).to.eql(expectPkg);
            });

        });

        describe('package.json exits', function(){
            before(function(){
                return init.initPkgJSON(def);
            });

            it('should create package.json', function(){
                expect(JSON.parse(fs.readFileSync(path.join(def.home, 'package.json')))).to.eql(expectPkg);
            });
        });
    });

    describe('initInfo', function(){
        var data = {};
        var saved = true;
        var def = {
            config: {
                set: function(key, value){
                    data[key] = value;
                    saved = false;
                },
                get: function(key){
                    return data[key]
                },
                save: function() {
                    return new Promise(function(resolve){

                        Promise.delay(10).then(function(){
                            saved = true;
                            resolve();
                        });
                    });
                }
            }
        };
        describe("not inited", function () {
            before(function(){
                return init.initInfo(def);
            });

            it("should save email and data to config", function () {
                expect(saved).to.be(true);
            });

            it("should generate mid", function () {
                expect(data.mid).to.be.a('string');
            });

            it("should save email", function () {
                expect(data.mid).to.be.a('string');
            });

            it("should save mail", function () {
                expect(data.mid).to.be.a('string');
            });
        });

        describe("not inited", function () {
            before(function(){
                saved = false;
                return init.initInfo(def);
            });

            it("should save email and data to config", function () {
                expect(saved).to.be(false);
            });

            it("should generate mid", function () {
                expect(data.mid).to.be.a('string');
            });

            it("should save email", function () {
                expect(data.mid).to.be.a('string');
            });

            it("should save mail", function () {
                expect(data.mid).to.be.a('string');
            });
        });

    });

    describe("dailyCheck", function () {
        var def = {
        };
    });

    describe('checkPlugin', function(){
        var def = {
            home: 'tmp/.def'
        };

    });


});