
var sinon = require('sinon');
var def = require('../lib/index');
var expect = require('expect.js');
var Promise = require('bluebird');

describe('plugin-runtime', function() {


    describe('def plugin system', function(){
        before(function(){
            this.request_ = def.request;
            def.request = Promise.resolve;
        });

        after(function(){
            def.request = this.request_;
        });

        beforeEach(function(){
            sinon.spy(def.log, 'warn');
            sinon.spy(def, 'request');
            this.Runtime = require('../lib/plugin-runtime')(def);
        });

        afterEach(function(){
            def.log.warn.restore();
            def.request.restore();
        });
        
        it('should warn when plugin pkg has no defDependencies when request some other plugin', function(){
            var that = { name: 'kimi' , pkg : {}};

            return this.Runtime.prototype.request.call(that, 'kimi-awp:say', { opt: 1 })
                .then(function(){
                    expect(def.log.warn.calledOnce).to.be.ok();
                    expect(def.request.calledOnce).to.be.ok();
                });

        });

        it('should not warn when defDependencies is present', function () {

            var that = { name: 'kimi' , pkg : {
                defDependencies: ['def-kimi-awp']
            }};

            return this.Runtime.prototype.request.call(that, 'kimi-awp:say', { opt: 1 })
                .then(function(){
                    expect(def.log.warn.calledOnce).to.not.be.ok();
                    expect(def.request.calledOnce).to.be.ok();
                });
        });

        it('should not warn when an @ali defDependencies is present', function () {

            var that = { name: 'kimi' , pkg : {
                defDependencies: ['@ali/def-kimi-awp']
            }};

            return this.Runtime.prototype.request.call(that, 'kimi-awp:say', { opt: 1 })
                .then(function(){
                    expect(def.log.warn.calledOnce).to.not.be.ok();
                    expect(def.request.calledOnce).to.be.ok();
                });
        });
    });
});